#include "rpg.h"
#include "mouse.h"

enum {R, G, B};

byte far *mainbuf;
tileType far *tiles;
paletteType palette;
char tileNames[NUM_TILES][9];
byte tileColor[NUM_TILES];
byte walkable[NUM_TILES];

tileType clipboard;
byte scratch[3][3];

int curTile = 0;
int Color = 0;

//////////////////////////////////////////////////////////////////////////////

void errorAbort(char *message)
{
  setvmode(0x3);
  cputs(message);
  if (getch() == 0) getch();
  exit(1);
}

//////////////////////////////////////////////////////////////////////////////

void Init(void)
{
  if (_argc < 4)
    errorAbort("Error:\n\n\rEDITART <artfile> <palettefile> <infofile>\n\r");

  int readTiles   = 0;
  int readPalette = 0;
  int readInfo    = 0;

  FILE *handle;
  handle = fopen(_argv[1], "rb");
  if (handle == NULL)
  {
    cprintf("Artfile %s does not exist.  Create it? (Y/N)",
            strupr(_argv[1]));
    if (toupper(getch()) == 'N')
      errorAbort("EditArt aborted\n\r");
  }
  else
    readTiles = 1;
  fclose(handle);

  handle = fopen(_argv[2], "rb");
  if (handle == NULL)
  {
    cprintf("\n\rPalettefile %s does not exist.  Create it? (Y/N)",
            strupr(_argv[2]));
    if (toupper(getch()) == 'N')
      errorAbort("EditArt aborted\n\r");
  }
  else
    readPalette = 1;
  fclose(handle);

  handle = fopen(_argv[3], "rb");
  if (handle == NULL)
  {
    cprintf("\n\rInfofile %s does not exist.  Create it? (Y/N)",
            strupr(_argv[3]));
    if (toupper(getch()) == 'N')
      errorAbort("EditArt aborted\n\r");;
  }
  else
    readInfo = 1;
  fclose(handle);

  setvmode(0x13);

  int buttons;
  if (!resetMouse(&buttons))
    errorAbort("Error: Mouse not detected\n\r");

  tiles = (tileType far *)farmalloc((long)sizeof(tileType) * NUM_TILES);
  if (tiles == NULL)
    errorAbort("Error: Not enough memory\n\r");

  mainbuf = (byte far *)farmalloc(64000L);
  if (mainbuf == NULL)
  {
    farfree(tiles);
    errorAbort("Error: Not enough memory\n\r");
  }

  showMouse();

  long x;

  if (readTiles)
  {
    handle = fopen(_argv[1], "rb");
    for (x = 0; x < (long)sizeof(tileType) * NUM_TILES; x++)
      ((unsigned char far *)tiles)[x] = fgetc(handle);
    fread(walkable, 1, NUM_TILES, handle);
    fclose(handle);
  }
  else
  {
    for (x = 0; x < (long)sizeof(tileType) * NUM_TILES; x++)
      ((byte far *)tiles)[x] = 0;
  }

  if (readPalette)
    loadPalette(_argv[2], palette);
  else
    getAllPalette(palette);
  setAllPalette(palette);

  if (readInfo)
  {
    handle = fopen(_argv[3], "rb");
    for (x = 0; x < NUM_TILES; x++)
      fread(tileNames[x], 9, 1, handle);
    fread(tileColor, NUM_TILES, 1, handle);
    fclose(handle);
  }
}

//////////////////////////////////////////////////////////////////////////////

void Close(void)
{
  hideMouse();
  farfree(mainbuf);
  farfree(tiles);
  setvmode(0x3);
}

//////////////////////////////////////////////////////////////////////////////

void drawMisc(void)
{
  hideMouse();
  clearArea(68, 186, 24, 8, pixel(0, 0));
  clearArea(164, 186, 24, 8, pixel(0, 0));

  char string[80];
  sprintf(string, "Tile: %3d  Color: %3d", curTile, Color);
  blitString(20, 186, 2, string);

  clearArea(190, 124, 96, 8, pixel(0, 0));
  sprintf(string, "R%2d G%2d B%2d", palette[Color].red, palette[Color].green,
                                    palette[Color].blue);
  blitString(190, 124, 2, string);

  clearArea(190, 136, 88, 8, pixel(0, 0));
  blitString(190, 136, 2, "N:");
  blitString(206, 136, 2, tileNames[curTile]);

  clearArea(190, 148, 20, 20, tileColor[curTile]);

  clearArea(238, 40, 32, 8, pixel(0, 0));
  blitString(238, 40, 2, (walkable[curTile] ? "Walk" : "Stop"));

  showMouse();
}

//////////////////////////////////////////////////////////////////////////////

void getTileName(void)
{
  char name[10] = {0,0,0,0,0,0,0,0,0};
  char c, index = 0;

  hideMouse();

  clearArea(206, 136, 72, 8, pixel(0, 0));
  blitChar(206, 136, 2, '_');

  while ((c = getch()) != 13)
  {
    if (c == 0) getch();

    if (c == 8 && index > 0)
      name[--index] = 0;

    if (c >= 32 && c <= 125 && index < 8)
      name[index++] = c;

    clearArea(206, 136, 72, 8, pixel(0, 0));
    name[index] = '_';
    blitString(206, 136, 2, name);
    name[index] = 0;
  }

  for (c = 0; c < 9; c++)
    tileNames[curTile][c] = name[c];

  showMouse();
}

//////////////////////////////////////////////////////////////////////////////

void getColorElem(int elem)
{
  char str[4] = {0,0,0,0};
  char c, index = 0;

  hideMouse();

  clearArea(198 + elem * 32, 124, 16, 8, pixel(0, 0));
  blitChar(198 + elem * 32, 124, 2, '_');

  while ((c = getch()) != 13)
  {
    if (c == 0) getch();

    if (c == 8 && index > 0)
      str[--index] = 0;

    if (c >= '0' && c <= '9' && index < 2)
      str[index++] = c;

    if (atoi(str) > 63)
    {
      str[0] = '6';
      str[1] = '3';
    }

    clearArea(198 + elem * 32, 124, 24, 8, pixel(0, 0));
    str[index] = '_';
    blitString(198 + elem * 32, 124, 2, str);
    str[index] = 0;
  }

  switch (elem)
  {
    case R: palette[Color].red   = atoi(str); break;
    case G: palette[Color].green = atoi(str); break;
    case B: palette[Color].blue  = atoi(str);
  }
  setPalette(Color, &palette[Color]);

  showMouse();
}

//////////////////////////////////////////////////////////////////////////////

void drawPalette(void)
{
  hideMouse();
  int x, y;
  int ix, iy;
  for (x = 0; x < 16; x++)
    for (y = 0; y < 16; y++)
      for (ix = 0; ix < 4; ix++)
        for (iy = 0; iy < 4; iy++)
          pixel(190 + x * 4 + ix, 56 + y * 4 + iy) = y * 16 + x;

  showMouse();
}

//////////////////////////////////////////////////////////////////////////////

int tileOffset, numpixels;
int startx, endx;
int starty, endy, iy;

void blitTile(int x, int y, byte tile)
{
  if (x < -19 || x > 320) return;
  if (y < -19 || y > 200) return;

  tileOffset = 0;
  if (x < 0)
  {
    startx = 0;
    tileOffset = -x;
  }
  else startx = x;

  if (y < 0)   starty = 0; else starty = y;
  if (x > 299) endx = 320; else endx = x + 20;
  if (y > 179) endy = 200; else endy = y + 20;

  numpixels = endx - startx;
  for (iy = starty; iy < endy; iy++)
    _fmemcpy(curBuf + (iy << 8) + (iy << 6) + startx,
             tiles[tile] + ((iy - y) * 20 + tileOffset),
             numpixels);
}

//////////////////////////////////////////////////////////////////////////////

void drawTile(void)
{
  hideMouse();

  int x, y;
  for (x = 0; x < 20; x++)
    for (y = 0; y < 20; y++)
    {
      int ix, iy;
      for (ix = 0; ix < 8; ix++)
        for (iy = 0; iy < 8; iy++)
          pixel(20 + x * 8 + ix, 20 + y * 8 + iy) =
            tiles[curTile][y * 20 + x];
    }

  showMouse();
}

//////////////////////////////////////////////////////////////////////////////

void drawScreen(void)
{
  hideMouse();

  long x;
  for (x = 0; x < 64000; x++)
    screen[x] = 19;

  blitString(20,  6,  2, "EditArt 2.0 - Chad Austin");
  blitString(190, 20, 2, "Quit  Copy");
  blitString(190, 30, 2, "Save  Paste");
  blitString(190, 40, 2, "Fill");

  blitString(200, 186, 2, "<--");
  blitString(240, 186, 2, "-->");

  showMouse();
}

//////////////////////////////////////////////////////////////////////////////

void drawScratch(void)
{
  hideMouse();
  int x, y;
  for (x = 0; x < 3; x++)
    for (y = 0; y < 3; y++)
      blitTile(258 + x * 20, 56 + y * 20, scratch[x][y]);
  showMouse();
}

//////////////////////////////////////////////////////////////////////////////

void saveTiles(void)
{
  FILE *handle;
  handle = fopen(_argv[1], "wb");
  if (handle == NULL)
  {
    Close();
    errorAbort("Error: Could not open artfile\n\r");
  }

  long x;
  for (x = 0; x < (long)(NUM_TILES) * 400; x++)
    fputc(((byte far *)tiles)[x], handle);
  fwrite(walkable, 1, NUM_TILES, handle);

  fclose(handle);

  handle = fopen(_argv[3], "wb");
  if (handle == NULL)
  {
    Close();
    errorAbort("Error: Could not open textfile\n\r");
  }

  for (x = 0; x < NUM_TILES; x++)
    fwrite(tileNames[x], 9, 1, handle);
  fwrite(tileColor, NUM_TILES, 1, handle);

  fclose(handle);
}

//////////////////////////////////////////////////////////////////////////////

void getMouseXYButtons(int *x, int *y, int *buttons)
{ mouseXYButtons(x, y, buttons); *x /= 2; }

//////////////////////////////////////////////////////////////////////////////

byte within(int x, int y, int x1, int y1, int x2, int y2)
{ if (x >= x1 && x <= x2 && y >= y1 && y <= y2) return 1; else return 0; }

//////////////////////////////////////////////////////////////////////////////

int main(void)
{
  Init();
  drawScreen();
  drawTile();
  drawPalette();
  drawMisc();
  drawScratch();

  int x, y, buttons;

  int done = 0;
  while (!done)
  {
    getMouseXYButtons(&x, &y, &buttons);

    if (buttons & 1)
    {
      if (within(x, y, 20, 20, 179, 179))
      {
        int firsttime = 1;

        while (buttons & 1)
        {
          int oldx = x, oldy = y;
          getMouseXYButtons(&x, &y, &buttons);

          if (within(x, y, 20, 20, 179, 179))
          {
            if (oldx != x || oldy != y || firsttime)
            {
              tiles[curTile][((y - 20) / 8) * 20 + ((x - 20) / 8)] = Color;
              drawTile();
              drawScratch();
              firsttime = 0;
            }
          }
        }
      }
      else
      {
        while (buttons & 1)
          getMouseXYButtons(&x, &y, &buttons);
      }

      int oldTile = curTile;
      if (within(x, y, 240, 180, 263, 199))
        if (++curTile == NUM_TILES) curTile = 0;

      if (within(x, y, 200, 180, 223, 199))
        if (--curTile == -1) curTile = NUM_TILES - 1;

      int oldColor = Color;
      if (within(x, y, 190, 56, 253, 119))
        Color = ((y - 56) / 4) * 16 + ((x - 190) / 4);

      if (oldTile != curTile || oldColor != Color)
        { drawTile(); drawMisc(); }

      if (within(x, y, 190, 148, 219, 167))
        { tileColor[curTile] = Color; drawMisc(); }

      int i;

      if (within(x, y, 238, 20, 238 + 31, 20 + 7))
        for (i = 0; i < 400; i++)
          clipboard[i] = tiles[curTile][i];

      if (within(x, y, 238, 30, 238 + 31, 30 + 7))
      {
        for (i = 0; i < 400; i++)
          tiles[curTile][i] = clipboard[i];
        drawTile();
        drawScratch();
      }

      if (within(x, y, 238, 40, 238 + 31, 40 + 7))
      {
        if (walkable[curTile])
          walkable[curTile] = 0;
        else
          walkable[curTile] = 1;
        drawMisc();
      }

      if (within(x, y, 190, 20, 190 + 31, 27))
        done = 1;

      if (within(x, y, 190, 30, 190 + 31, 37))
        { saveTiles(); savePalette(_argv[2], palette); }

      if (within(x, y, 190, 40, 190 + 31, 47))
      {
        int index;
        for (index = 0; index < 400; index++)
          tiles[curTile][index] = Color;
        drawTile();
        drawScratch();
      }

      if (within(x, y, 190, 124, 190 + 8 * 12 - 1, 124 + 7))
      {
        if (x >= 190 && x < 190 + 24) getColorElem(R);
        if (x >= 222 && x < 222 + 24) getColorElem(G);
        if (x >= 254 && x < 254 + 24) getColorElem(B);
        drawMisc();
      }

      if (within(x, y, 190, 136, 190 + 79, 136 + 7))
      {
        getTileName();
        drawMisc();
      }

      if (within(x, y, 258, 56, 258 + 59, 56 + 59))
      {
        scratch[(x - 256) / 20][(y - 56) / 20] = curTile;
        drawScratch();
      }
    }

    if (buttons & 2)
    {
      while (buttons & 2)
        getMouseXYButtons(&x, &y, &buttons);

      if (within(x, y, 20, 20, 179, 179))
      {
        Color = tiles[curTile][((y - 20) / 8) * 20 + ((x - 20) / 8)];
        drawMisc();
      }
    }

    int i;

    if (kbhit())
    {
#define ct tiles[curTile]

      switch (getch())
      {
        case 0:
          switch (getch())
          {
            case 75:
              for (y = 0; y < 20; y++)
                for (x = 0; x < 19; x++)
                  ct[y * 20 + x] = ct[y * 20 + x + 1];
              break;

            case 77:
              for (y = 0; y < 20; y++)
                for (x = 19; x > 0; x--)
                  ct[y * 20 + x] = ct[y * 20 + x - 1];
              break;

            case 72:
              for (x = 0; x < 20; x++)
                for (y = 0; y < 19; y++)
                  ct[y * 20 + x] = ct[(y + 1) * 20 + x];
              break;

            case 80:
              for (x = 0; x < 20; x++)
                for (y = 19; y > 0; y--)
                  ct[y * 20 + x] = ct[(y - 1) * 20 + x];
              break;
          }
          drawMisc();
          drawTile();
          drawScratch();
          break;

        case ',': case '<': if (--curTile == -1) curTile = NUM_TILES - 1;
                            drawTile(); drawMisc(); break;
        case '.': case '>': if (++curTile == NUM_TILES) curTile = 0;
                            drawTile(); drawMisc(); break;

        case 'q': case 'Q':
          done = 1;
          break;

        case 's': case 'S':
          saveTiles();
          savePalette(_argv[2], palette);
          break;

        case 'f': case 'F':
          for (i = 0; i < 400; i++)
            tiles[curTile][i] = Color;
          drawTile();
          drawScratch();
          break;

        case 'c': case 'C':
          for (i = 0; i < 400; i++)
            clipboard[i] = tiles[curTile][i];
          break;

        case 'p': case 'P':
          for (i = 0; i < 400; i++)
            tiles[curTile][i] = clipboard[i];
          drawTile();
          drawScratch();
          break;
      }
    }
  }

  Close();
  return 0;
}
