#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <alloc.h>
#include <string.h>
#include <ctype.h>

#include "rpg.h"
#include "mouse.h"

#define map(x, y) (Map          [(y) * MAP_X_SIZE + (x)])
#define bol(x, y) (battleOverlay[(y) * MAP_X_SIZE + (x)])

byte far *Map;
byte far *battleOverlay;
byte battleColors[NUM_BATTLES];
char battleNames[NUM_BATTLES][9];
tileType far *tiles;
char tileNames[NUM_TILES][9];
byte tileColor[NUM_TILES];
paletteType palette;

int brush;
int block;
int curTile;
int curBattle;
int zoomFactor;
int mapx, mapy;

//////////////////////////////////////////////////////////////////////////////

void errorAbort(char *message)
{
  cputs(message);
  if (getch() == 0) getch();
  exit(1);
}

//////////////////////////////////////////////////////////////////////////////

void Init(void)
{
  if (_argc < 5)
    errorAbort("Usage:  EDITMAP <mapfile> <artfile> <infofile> <palettefile>\n\r");

  int buttons;
  if (!resetMouse(&buttons))
    errorAbort("Error: Mouse not detected\n\r");
    
  Map = (byte far *)farmalloc(MAP_X_SIZE * MAP_Y_SIZE);
  if (Map == NULL)
    errorAbort("Error: Not enough memory\n\r");

  battleOverlay = (byte far *)farmalloc(MAP_X_SIZE * MAP_Y_SIZE);
  if (battleOverlay == NULL)
  {
    farfree(Map);
    errorAbort("Error: Not enough memory\n\r");
  }

  tiles = (tileType far *)farmalloc(sizeof(tileType) * NUM_TILES);
  if (tiles == NULL)
  {
	 farfree(Map);
    farfree(battleOverlay);
    errorAbort("Error: Not enough memory\n\r");
  }

  setvmode(0x13);

  FILE *handle;
  long x;

  int readmap = 1;
  handle = fopen(_argv[1], "rb");
  if (handle == NULL)
  {
    setvmode(0x3);
    cprintf("Error: Could not open mapfile\n\r"
            "Create %s? (Y/N)", strupr(_argv[1]));
    if (toupper(getche()) == 'N')
    {
      farfree(Map);
      farfree(battleOverlay);
      farfree(tiles);
      errorAbort("\n\rEditMap aborted\n\r");
    }
	 cputs("\n\r");
    readmap = 0;
    setvmode(0x13);
  }

  if (readmap)
  {
    for (x = 0; x < MAP_X_SIZE * MAP_Y_SIZE; x++)
      Map[x] = fgetc(handle);
    for (x = 0; x < MAP_X_SIZE * MAP_Y_SIZE; x++)
      battleOverlay[x] = fgetc(handle);
    fclose(handle);
  }
  else
    for (x = 0; x < MAP_X_SIZE * MAP_Y_SIZE; x++)
    {
      Map[x] = 0;
      battleOverlay[x] = 0;
    }

  handle = fopen(_argv[2], "rb");
  if (handle == NULL)
  {
    farfree(Map);
	 farfree(battleOverlay);
    farfree(tiles);
    setvmode(0x3);
    errorAbort("Error: Could not open artfile\n\r");
  }

  for (x = 0; x < sizeof(tileType) * NUM_TILES; x++)
    ((byte far *)tiles)[x] = fgetc(handle);
  fclose(handle);

  handle = fopen(_argv[3], "rb");
  if (handle == NULL)
  {
    farfree(Map);
    farfree(battleOverlay);
    farfree(tiles);
    setvmode(0x3);
    errorAbort("Error: Could not open infofile\n\r");
  }

  for (x = 0; x < NUM_TILES; x++)
    fread(tileNames[x], 9, 1, handle);
  fread(tileColor, NUM_TILES, 1, handle);

  fclose(handle);

  handle = fopen("b_info.dat", "rb");
  if (handle == NULL)
  {
    farfree(Map);
    farfree(battleOverlay);
    farfree(tiles);
    setvmode(0x3);
    errorAbort("Error: Could not open battle info file (B_INFO.DAT)\n\r");
  }
  fread(battleColors, 1, NUM_BATTLES, handle);
  fread(battleNames, 9, NUM_BATTLES, handle);
  fclose(handle);

  if (loadPalette(_argv[4], palette))
  {
    farfree(Map);
    farfree(battleOverlay);
    farfree(tiles);
    setvmode(0x3);
    errorAbort("Error: Could not open palettefile\n\r");
  }

  setAllPalette(palette);
  showMouse();
}

//////////////////////////////////////////////////////////////////////////////

void Close(void)
{
  hideMouse();
  setvmode(0x3);
  farfree(Map);
  farfree(tiles);
}

//////////////////////////////////////////////////////////////////////////////

void drawScreen(void)
{
  byte far *mainbuf;

  mainbuf = (byte far *)farmalloc(64000L);
  if (mainbuf == NULL)
  {
    Close();
	 errorAbort("Error: Not enough memory\n\r");
  }

  hideMouse();
  curBuf = mainbuf;

  long x;
  for (x = 0; x < 64000; x++)
    curBuf[x] = 19;

  blitString(220, 10,  2, "EditMap 2.5");
  blitString(220, 20,  2, "Chad Austin");
  blitString(220, 40, 2, "Quit  Save");
  blitString(220, 50, 2, "Z In  Z Out");
  blitString(220, 60, 2, "Fill");
  blitString(220, 150, 2, "<--  -->");
  blitString(220, 190, 2, "<--  -->");
  blitScreen(mainbuf);
  showMouse();
  curBuf = screen;

  farfree(mainbuf);
}

//////////////////////////////////////////////////////////////////////////////

void saveMap(void)
{
  FILE *handle;
  handle = fopen(_argv[1], "wb");
  if (handle == NULL)
  {
    Close();
    errorAbort("Error: Could not write to map file\n\r");
  }

  long x;
  for (x = 0; x < MAP_X_SIZE * MAP_Y_SIZE; x++)
    fputc(Map[x], handle);
  for (x = 0; x < MAP_X_SIZE * MAP_Y_SIZE; x++)
    fputc(battleOverlay[x], handle);

  fclose(handle);
}

//////////////////////////////////////////////////////////////////////////////

void getMouseXYButtons(int *x, int *y, int *buttons)
{
  mouseXYButtons(x, y, buttons);
  *x /= 2;
}

//////////////////////////////////////////////////////////////////////////////

#define xmax (xmin + xoff)
#define ymax (ymin + yoff)

byte within(int x, int y, int xmin, int ymin, int xoff, int yoff)
{
  if (x >= xmin && x <= xmax && y >= ymin && y <= ymax)
    return 1;

  return 0;
}

//////////////////////////////////////////////////////////////////////////////

void fillMap(void)
{
  int i;
  if (brush)
	 for (i = 0; i < MAP_X_SIZE * MAP_Y_SIZE; i++)
      battleOverlay[i] = curBattle;
  else
    for (i = 0; i < MAP_X_SIZE * MAP_Y_SIZE; i++)
      Map[i] = curTile;
}

//////////////////////////////////////////////////////////////////////////////

void zoom_in(void)
{
  int oldZFactor = zoomFactor;
  if (++zoomFactor == 4) zoomFactor = 3;
  if (oldZFactor != zoomFactor)
  {
    switch (zoomFactor)
    {
      case 1: mapx = 24; mapy = 24; break;
      case 2: mapx += 15; mapy += 15; break;
      case 3: mapx += 5; mapy += 5; break;
    }
  }
}

//////////////////////////////////////////////////////////////////////////////

void zoom_out(void)
{
  int oldZFactor = zoomFactor;
  if (--zoomFactor == -1) zoomFactor = 0;
  if (oldZFactor != zoomFactor)
  {
    switch (zoomFactor)
    {
      case 2:
        mapx -= 5; if (mapx < 0) mapx = 0; if (mapx > 80) mapx = 80;
        mapy -= 5; if (mapy < 0) mapy = 0; if (mapy > 80) mapy = 80;
        break;

      case 1:
        mapx -= 15; if (mapx < 0) mapx = 0; if (mapx > 50) mapx = 50;
        mapy -= 15; if (mapy < 0) mapy = 0; if (mapy > 50) mapy = 50;
        break;

      case 0: mapx = 0;  mapy = 0;  break;
    }
  }
}

//////////////////////////////////////////////////////////////////////////////

void drawMapOnly(void)
{
  int x,  y;
  int ix, iy;
  int tx, ty;

  switch (zoomFactor)
  {
    case 0:
      for (x = 0; x < 100; x++)
        for (y = 0; y < 100; y++)
        {
          tx = x << 1; ty = y << 1;
          pixel(tx,     ty    ) =
          pixel(tx + 1, ty    ) =
          pixel(tx,     ty + 1) =
          pixel(tx + 1, ty + 1) = tileColor[map(x, y)];
        }
      break;

    case 1:
		for (x = 0; x < 50; x++)
        for (y = 0; y < 50; y++)
          for (ix = 0; ix < 4; ix++)
            for (iy = 0; iy < 4; iy++)
              pixel((x << 2) + ix, (y << 2) + iy) =
                tileColor[map(mapx + x, mapy + y)];
      break;

    case 2:
      for (x = 0; x < 20; x++)
        for (y = 0; y < 20; y++)
          for (ix = 0; ix < 10; ix++)
            for (iy = 0; iy < 10; iy++)
              if ((bol(mapx + x, mapy + y) == 0) || (ix + iy) % 2 == 0 || !brush)
                pixel(x * 10 + ix, y * 10 + iy) =
                  tiles[map(mapx + x, mapy + y)][iy * 40 + ix * 2];
              else
                pixel(x * 10 + ix, y * 10 + iy) =
                  battleColors[bol(mapx + x, mapy + y)];
      break;

    case 3:
      for (x = 0; x < 10; x++)
        for (y = 0; y < 10; y++)
			 for (ix = 0; ix < 20; ix++)
            for (iy = 0; iy < 20; iy++)
              if ((bol(mapx + x, mapy + y) == 0) || (ix + iy) % 2 == 0 || !brush)
                pixel(x * 20 + ix, y * 20 + iy) =
                  tiles[map(mapx + x, mapy + y)][iy * 20 + ix];
              else
                pixel(x * 20 + ix, y * 20 + iy) =
                  battleColors[bol(mapx + x, mapy + y)];
      break;
  }
}

//////////////////////////////////////////////////////////////////////////////

void drawMap(void)
{  
  hideMouse();

  drawMapOnly();

  int x, y;

  char string[10];
  sprintf(string, "Zoom: %d", zoomFactor);

  clearArea(220, 90, 80, 8, 19);
  blitString(220, 90, 2, string);

  for (x = 0; x < 20; x++)
    for (y = 0; y < 20; y++)
      pixel(x + 220, y + 120) = battleColors[curBattle];

  clearArea(250, 126, 64, 8, 19);
  blitString(250, 126, 2, battleNames[curBattle]);

  for (x = 0; x < 20; x++)
    for (y = 0; y < 20; y++)
      pixel(x + 220, y + 165) = tiles[curTile][y * 20 + x];

  clearArea(250, 171, 64, 8, 19);
  blitString(250, 171, tileColor[curTile], tileNames[curTile]);

  clearArea(220, 105, 96, 8, 19);
  blitString(220, 105, 2, (brush ? "Battle Types" : "Tiles"));

  clearArea(268, 60, 32, 8, 19);
  if (block == 0) blitString(268, 60, 2, "Tile");
  if (block == 1) blitString(268, 60, 2, "3x3");
  if (block == 2) blitString(268, 60, 2, "5x5");

  showMouse();      
}

//////////////////////////////////////////////////////////////////////////////

void printXYValues(int x, int y)
{
  if (x < 200)
  {
    char string[6];

    hideMouse();
    switch (zoomFactor)
    {
      case 0: sprintf(string, "%2d %2d", x / 2, y / 2); break;
      case 1: sprintf(string, "%2d %2d", x / 4 + mapx, y / 4 + mapy); break;
      case 2: sprintf(string, "%2d %2d", x / 10 + mapx, y / 10 + mapy); break;
      case 3: sprintf(string, "%2d %2d", x / 20 + mapx, y / 20 + mapy); break;
    }
    clearArea(220, 80, 48, 8, 19);
    blitString(220, 80, 2, string);
    showMouse();
  }
}

//////////////////////////////////////////////////////////////////////////////

void setTile(int x, int y, int tile)
{
  int ix, iy;
  if (block == 0)
    map(x, y) = tile;
  else
    if (block == 1)
    {
      for (ix = x - 1; ix <= x + 1; ix++)
        for (iy = y - 1; iy <= y + 1; iy++)
          if (within(ix, iy, 0, 0, 99, 99)) map(ix, iy) = tile;
    }
    else
      for (ix = x - 2; ix <= x + 2; ix++)
        for (iy = y - 2; iy <= y + 2; iy++)
          if (within(ix, iy, 0, 0, 99, 99)) map(ix, iy) = tile;
}

void setBattle(int x, int y, int battle)
{
  int ix, iy;
  if (block == 0)
    bol(x, y) = battle;
  else
    if (block == 1)
    {
      for (ix = x - 1; ix <= x + 1; ix++)
        for (iy = y - 1; iy <= y + 1; iy++)
          if (within(ix, iy, 0, 0, 99, 99)) bol(ix, iy) = battle;
    }
    else
      for (ix = x - 2; ix <= x + 2; ix++)
        for (iy = y - 2; iy <= y + 2; iy++)
          if (within(ix, iy, 0, 0, 99, 99)) bol(ix, iy) = battle;
}

//////////////////////////////////////////////////////////////////////////////

int main(void)
{
  int x = 0, y = 0, buttons;
  int oldx, oldy;

  Init();
  drawScreen();
  drawMap();

  int done = 0;
  while (!done)
  {
	  oldx = x; oldy = y;

    getMouseXYButtons(&x, &y, &buttons);

    if (x != oldx || y != oldy)
      printXYValues(x, y);

    if (buttons & 1)
    {
      if (within(x, y, 0, 0, 199, 199))
      {
        oldx = 320; oldy = 200;

        while (buttons & 1)
        {
          getMouseXYButtons(&x, &y, &buttons);

          if (within(x, y, 0, 0, 199, 199))
          {
            if (x != oldx || y != oldy)
            {
              hideMouse();

              switch (zoomFactor)
              {
                case 0:
                  if (brush == 0)
                  {
                    setTile(x / 2, y / 2, curTile);
                    drawMapOnly();
                  }
                  break;

                case 1:
                  if (brush == 0)
                  {
                    setTile(mapx + x / 4, mapy + y / 4, curTile);
                    drawMapOnly();
                  }
                  break;

                case 2:
                  if (brush == 0)
                    setTile(mapx + x / 10, mapy + y / 10, curTile);
                  else
                    setBattle(mapx + x / 10, mapy + y / 10, curBattle);
                  drawMap();
                  break;

                case 3:
                  if (brush == 0)
                    setTile(x / 20 + mapx, y / 20 + mapy, curTile);
                  else
                    setBattle(x / 20 + mapx, y / 20 + mapy, curBattle);
                  drawMap();
                  break;
              }

              showMouse();
              printXYValues(x, y);
            }
          }

          oldx = x;
          oldy = y;
        }
        drawMap();
      }

      if (within(x, y, 220, 40, 32, 8)) done = 1;
      if (within(x, y, 268, 40, 32, 8)) saveMap();
      if (within(x, y, 220, 50, 32, 8)) zoom_in();
      if (within(x, y, 268, 50, 40, 8)) zoom_out();
      if (within(x, y, 220, 60, 32, 8)) fillMap();

      if (within(x, y, 220, 150, 24, 8))
        if (--curBattle == -1)
          curBattle = NUM_BATTLES - 1;

      if (within(x, y, 260, 150, 24, 8))
        if (++curBattle == NUM_BATTLES)
          curBattle = 0;

      if (within(x, y, 220, 190, 24, 8))
        if (--curTile == -1)
          curTile = NUM_TILES - 1;

      if (within(x, y, 260, 190, 24, 8))
        if (++curTile == NUM_TILES)
          curTile = 0;

      if (within(x, y, 220, 105, 96, 8))
        if (brush) brush = 0; else brush = 1;

      if (within(x, y, 268, 60, 32, 8))
      {
        if (block == 0) block = 1; else
        if (block == 1) block = 2; else
        if (block == 2) block = 0;
      }

      printXYValues(x, y);
      drawMap();

      while (buttons & 1)
        getMouseXYButtons(&x, &y, &buttons);
    }

    if (buttons & 2)
    {
      if (within(x, y, 0, 0, 199, 199))
        if (brush)
          switch (zoomFactor)
          {
            case 2: curBattle = bol(x / 10 + mapx, y / 10 + mapy); break;
            case 3: curBattle = bol(x / 20 + mapx, y / 20 + mapy); break;
          }
        else
          switch (zoomFactor)
          {
            case 0: curTile = map(x / 2, y / 2); break;
            case 1: curTile = map(x / 4 + mapx, y / 4 + mapy); break;
            case 2: curTile = map(x / 10 + mapx, y / 10 + mapy); break;
            case 3: curTile = map(x / 20 + mapx, y / 20 + mapy); break;
          }

      printXYValues(x, y);
      drawMap();

      while (buttons & 2)
        getMouseXYButtons(&x, &y, &buttons);
    }

    if (kbhit())
	 {
      switch (getch())
      {
        case 0:
          switch (zoomFactor)
          {
            case 1:
              switch (getch())
				  {
                case 75: if (--mapx == -1) mapx = 0; break;
                case 77: if (++mapx == 51) mapx = 50; break;
                case 72: if (--mapy == -1) mapy = 0; break;
                case 80: if (++mapy == 51) mapy = 50; break;
				  }
              break;
            case 2:
              switch (getch())
              {
                case 75: if (--mapx == -1) mapx = 0; break;
                case 77: if (++mapx == 81) mapx = 80; break;
                case 72: if (--mapy == -1) mapy = 0; break;
                case 80: if (++mapy == 81) mapy = 80; break;
              }
              break;
            case 3:
              switch (getch())
              {
                case 75: if (--mapx == -1) mapx = 0; break;
                case 77: if (++mapx == 91) mapx = 90; break;
                case 72: if (--mapy == -1) mapy = 0; break;
                case 80: if (++mapy == 91) mapy = 90; break;
              }
			 }
          break;

        case 'q': case 'Q': done = 1; break;
        case 's': case 'S': saveMap(); break;
		  case '+': case '=': zoom_in(); break;
        case '-': case '_': zoom_out(); break;
        case 'f': case 'F': fillMap(); break;
        case ',': case '<':
          if (--curTile == -1) curTile = NUM_TILES - 1;
          break;
        case '.': case '>':
          if (++curTile == NUM_TILES) curTile = 0;
          break;
        case ';': case ':':
          if (--curBattle == -1) curBattle = NUM_BATTLES - 1;
          break;
        case '\'': case '\"':
          if (++curBattle == NUM_BATTLES) curBattle = 0;
          break;          
      }

      printXYValues(x, y);
      drawMap();
	 }
  }

  Close();
  return 0;
}
