#include "rpg.h"
#include "mouse.h"

#define TEMPFILE "temp.tmp"

#define pause() if (getch() == 0) getch()

byte *monsterImage;
byte *clipboard;
paletteType palette;
FILE *tempfile;
int Color;
int curImage;
int brush;

void copyfiles(char *file1, char *file2)
{
  FILE *stream1 = fopen(file1, "rb");
  FILE *stream2 = fopen(file2, "wb");

  int c;
  while ((c = fgetc(stream1)) != EOF)
    fputc(c, stream2);

  fclose(stream1);
  fclose(stream2);
}

void saveTile(char *file, long slot)
{
  FILE *stream = fopen(file, "r+b");
  fseek(stream, slot * 80 * 80, SEEK_SET);
  fwrite(monsterImage, 1, 80 * 80, stream);
  fclose(stream);
}

void loadTile(char *file, long slot)
{
  FILE *stream = fopen(file, "r+b");
  fseek(stream, slot * 80 * 80, SEEK_SET);
  fread(monsterImage, 1, 80 * 80, stream);
  fclose(stream);
}

void errorAbort(char *message)
{
  cputs(message);
  pause();
  Close();
  exit(1);
}

void Init(void)
{
  int buttons;
  if (!resetMouse(&buttons))
  {
    cputs("Error: EDITMPIC requires a mouse\n\r");
    pause();
    exit(1);
  }

  if (_argc < 3)
  {
    cputs("Usage: EDITMPIC monsterfile palettefile\n\r");
    pause();
    exit(1);
  }

  if (loadPalette(_argv[2], palette))
  {
    cputs("Error: Could not load palette file\n\r");
    pause();
    exit(1);
  }

  int readImages = 0;
  tempfile = fopen(_argv[1], "rb");
  if (tempfile != NULL)
    { readImages = 1; fclose(tempfile); }

  tempfile = fopen(TEMPFILE, "wb");
  if (tempfile == NULL)
  {
    cputs("Error: Could not create temp file\n\r");
    pause();
    exit(1);
  }
  fclose(tempfile);

  monsterImage = (byte *)malloc(80 * 80);
  if (monsterImage == NULL)
  {
    cputs("Error: Not enough memory\n\r");
    pause();
    exit(1);
  }
  memset(monsterImage, 0, 80 * 80);

  clipboard = (byte *)malloc(80 * 80);
  if (clipboard == NULL)
  {
    farfree(monsterImage);
    cputs("Error: Not enough memory\n\r");
    pause();
    exit(1);
  }
  memset(clipboard, 0, 80 * 80);

  cprintf("Loading, please wait...\n\r");
  if (readImages)
  {
    copyfiles(_argv[1], TEMPFILE);
    loadTile(TEMPFILE, 0);
  }
  else
  {
    tempfile = fopen(TEMPFILE, "wb");
    int x;
    for (x = 0; x < NUM_MONSTERS; x++)
      fwrite(monsterImage, 1, 80 * 80, tempfile);
    fclose(tempfile);
  }

  setvmode(0x13);
  setAllPalette(palette);
  showMouse();
}

void Close(void)
{
  hideMouse();
  setvmode(0x3);
  free(clipboard);
  free(monsterImage);

  char string[128];
  sprintf(string, "del %s", TEMPFILE);
  system(string);
}

void drawScreen(void)
{
  hideMouse();

  clearScreen(19);
  blitString(8, 6, 2, "EditMPic 1.1 - Chad Austin");

  blitString(200, 20, 2, "Quit  Save");
  blitString(200, 30, 2, "Copy  Paste");
  blitString(200, 40, 2, "Fill");

  blitString(200, 186, 2, "<--");
  blitString(232, 186, 2, "-->");
  
  showMouse();
}

#define PAL_X 200
#define PAL_Y 60

void drawPalette(void)
{
  hideMouse();
  int x, y;
  for (x = 0; x < 16; x++)
    for (y = 0; y < 16; y++)
      clearArea(x * 4 + PAL_X, y * 4 + PAL_Y, 4, 4, y * 16 + x);
  showMouse();
}

void drawMonster(void)
{
  hideMouse();
  int x, y;
  for (x = 0; x < 80; x++)
    for (y = 0; y < 80; y++)
    {
      pixel(x * 2 + 20, y * 2 + 20) =
      pixel(x * 2 + 21, y * 2 + 20) =
      pixel(x * 2 + 20, y * 2 + 21) =
      pixel(x * 2 + 21, y * 2 + 21) = monsterImage[y * 80 + x];
    }
  showMouse();
}

void drawMisc(void)
{
  hideMouse();
  char string[41];

  clearArea(20, 186, 80, 8, 19);
  sprintf(string, "Image: %d", curImage);
  blitString(20, 186, 2, string);

  clearArea(108, 186, 80, 8, 19);
  sprintf(string, "Color: %d", Color);
  blitString(108, 186, 2, string);

  clearArea(248, 40, 24, 8, 19);
  if (brush == 0) blitString(248, 40, 2, "1x1");
  if (brush == 1) blitString(248, 40, 2, "3x3");
  if (brush == 2) blitString(248, 40, 2, "5x5");
  showMouse();
}

void changeImage(int i)
{
  saveTile(TEMPFILE, curImage);

  curImage += i;
  if (curImage < 0) curImage = NUM_MONSTERS - 1;
  if (curImage > NUM_MONSTERS - 1) curImage = 0;

  loadTile(TEMPFILE, curImage);

  drawMonster();
  drawMisc();
}

void saveImages(void)
{
  hideMouse();
  saveTile(TEMPFILE, curImage);
  copyfiles(TEMPFILE, _argv[1]);
  showMouse();
}

void getMouseXYButtons(int *x, int *y, int *buttons)
{
  mouseXYButtons(x, y, buttons);
  *x /= 2;
}

byte within(int x, int y, int x1, int y1, int x2, int y2)
{ if (x >= x1 && x < x2 && y >= y1 && y < y2) return 1; else return 0; }

void setPixel(int x, int y, int color)
{
  int ix, iy;
  if (brush == 0) monsterImage[y * 80 + x] = color;

  if (brush == 1)
    for (ix = x - 1; ix < x + 2; ix++)
      for (iy = y - 1; iy < y + 2; iy++)
        if (within(ix, iy, 0, 0, 80, 80))
          monsterImage[iy * 80 + ix] = color;

  if (brush == 2)
    for (ix = x - 2; ix < x + 3; ix++)
      for (iy = y - 2; iy < y + 3; iy++)
        if (within(ix, iy, 0, 0, 80, 80))
          monsterImage[iy * 80 + ix] = color;
}

int main(void)
{
  Init();

  drawScreen();
  drawPalette();
  drawMonster();
  drawMisc();

  int x, y, buttons;

  int done = 0;
  while (!done)
  {
    getMouseXYButtons(&x, &y, &buttons);

    if (buttons & 1)
    {
      if (within(x, y, 20, 20, 180, 180))
      {
        setPixel((x - 20) / 2, (y - 20) / 2, Color);
        drawMonster();

        while (buttons & 1)
        {
          int oldx = x, oldy = y;
          getMouseXYButtons(&x, &y, &buttons);
          if (oldx != x || oldy != y)
          {
            if (within(x, y, 20, 20, 180, 180))
            {
              setPixel((x - 20) / 2, (y - 20) / 2, Color);
              drawMonster();
            }
          }
        }
      }

      while (buttons & 1)
        getMouseXYButtons(&x, &y, &buttons);

      if (within(x, y, PAL_X, PAL_Y, PAL_X + 64, PAL_Y + 64))
        { Color = ((y - PAL_Y) / 4) * 16 + (x - PAL_X) / 4; drawMisc(); }

      if (within(x, y, 200, 20, 232, 28)) done = 1;
      if (within(x, y, 248, 20, 270, 28)) saveImages();
      if (within(x, y, 200, 30, 232, 38))
        memcpy(clipboard, monsterImage, 80 * 80);
      if (within(x, y, 248, 30, 278, 38))
        { memcpy(monsterImage, clipboard, 80 * 80); drawMonster(); }
      if (within(x, y, 200, 40, 232, 48))
        { memset(monsterImage, Color, 80 * 80); drawMonster(); }
      if (within(x, y, 248, 40, 270, 48))
        { if (++brush == 3) brush = 0; drawMisc(); }

      if (within(x, y, 200, 186, 224, 194)) changeImage(-1);
      if (within(x, y, 232, 186, 256, 194)) changeImage(1);
    }

    if (buttons & 2)
    {
      while (buttons & 2)
        getMouseXYButtons(&x, &y, &buttons);

      if (within(x, y, 20, 20, 180, 180))
      {
        Color = monsterImage[(y - 20) / 2 * 80 + (x - 20) / 2];
        drawMisc();
      }
    }

    if (kbhit())
    {
      switch (toupper(getch()))
      {
        case 0:
          int ix, iy;
          switch (getch())
          {
            case 75:
              for (ix = 0; ix < 79; ix++)
                for (iy = 0; iy < 80; iy++)
                  monsterImage[iy * 80 + ix] = monsterImage[iy * 80 + ix + 1];
              break;

            case 72:
              for (iy = 0; iy < 79; iy++)
                for (ix = 0; ix < 80; ix++)
                  monsterImage[iy * 80 + ix] = monsterImage[(iy + 1) * 80 + ix];
              break;

            case 77:
              for (ix = 79; ix > 0; ix--)
                for (iy = 0; iy < 80; iy++)
                  monsterImage[iy * 80 + ix] = monsterImage[iy * 80 + ix - 1];
              break;

            case 80:
              for (iy = 79; iy > 0; iy--)
                for (ix = 0; ix < 80; ix++)
                  monsterImage[iy * 80 + ix] = monsterImage[(iy - 1) * 80 + ix];
              break;

          }
          drawMonster();
          break;

        case 'S': saveImages(); break;
        case 'Q': done = 1; break;
        case 'F': memset(monsterImage, Color, 80 * 80); drawMonster(); break;
        case 'C': memcpy(clipboard, monsterImage, 80 * 80); break;
        case 'P': memcpy(monsterImage, clipboard, 80 * 80); drawMonster(); break;
        case ',': case '<': changeImage(-1); break;
        case '.': case '>': changeImage(1);  break;
        case 'B': if (++brush == 3) brush = 0; drawMisc(); break;
      }
    }
  }

  Close();
}
