#include "rpg.h"

#define START 1
#define LOAD  0

extern byte far *ROMCharSet;

void blitCoolChar(int xc, int yc, char c)
{
  int offset, x, y;
  byte far *workChar;
  byte bitMask = 0x80;

  workChar = ROMCharSet + (c << 3);

  offset = (yc << 8) + (yc << 6) + xc;

  for (y = 0; y < 8; y++)
  {
    bitMask = 0x80;
    for (x = 0; x < 8; x++)
    {
      if (*workChar & bitMask)
        curBuf[offset + x] = y + (256 - 8);

      bitMask >>= 1;
    }

    offset += 320;
    workChar++;
  }
}

void blitCoolString(int x, int y, char *text)
{
  int index;
  for (index = 0; text[index]; index++)
    blitCoolChar(x + (index << 3), y, text[index]);
}

void HDelay(long msecs)
{
  HTimer tt;
  tt.timerOn();
  while (tt.getElapsed() < msecs) {}
  tt.timerOff();
}

int additem(int type, int num)
{
  int x;
  for (x = 0; x < NUM_ITEMS; x++)
  {
    if (gamedata.items[x].type == BLANK)
    {
      gamedata.items[x].type = type;
      gamedata.items[x].num  = num;
      return 0;
    }
  }
  return 1;
}

void PlaySound(int num)
{
  if (sb_exist && gameconfig.play_sound)
    start_sound(sounds[num], num, 0);
}

#define INTRO_FADE 10000L

void intro(void)
{
  PCX_PICTURE image;

  if (PCX_Init(&image))
    errorAbort("Error: Not enough memory\n\r");
  if (PCX_Load(WARNING_FILE, &image))
    { PCX_Close(&image); errorAbort("Error: Could not open intro file\n\r"); }

  colorType blank = { 0, 0, 0 };
  for (int index = 0; index < 256; index++)
    setPalette(index, &blank);

  blitScreen(image.buffer);
  PCX_Close(&image);

  fadeIn((colorType *)image.palette, INTRO_FADE);
  while (!keyPressed()) {}
  PlaySound(SOUND_SELECT);
  fadeOut((colorType *)image.palette, INTRO_FADE);

  if (PCX_Init(&image))
    errorAbort("Error: Not enough memory\n\r");
  if (PCX_Load(INTRO_FILE, &image))
    { PCX_Close(&image); errorAbort("Error: Could not open intro file\n\r"); }

  blitScreen(image.buffer);
  PCX_Close(&image);

  fadeIn((colorType *)image.palette, INTRO_FADE);
  while (!keyPressed()) {}
  PlaySound(SOUND_SELECT);
  fadeOut((colorType *)image.palette, INTRO_FADE);

  if (PCX_Init(&image))
    errorAbort("Error: Not enough memory\n\r");
  if (PCX_Load(INTRO2_FILE, &image))
    { PCX_Close(&image); errorAbort("Error: Could not open second intro file\n\r"); }

  blitScreen(image.buffer);
  PCX_Close(&image);

  fadeIn((colorType *)image.palette, INTRO_FADE);
  while (keyPressed()) {}
  while (!keyPressed()) {}
  PlaySound(SOUND_SELECT);
  fadeOut((colorType *)image.palette, INTRO_FADE);
}

//////////////////////////////////////////////////////////////////////////////

#define CFG_X 40
#define CFG_Y 40

void configure(void)
{
  clearScreen();
  window(10, 10, 319 - 20, 199 - 20);
  blitImage(CFG_X - 16, CFG_Y - 4, &arrow);
  blitCoolString(CFG_X, CFG_Y     , "Delay Rate:");
  blitCoolString(CFG_X, CFG_Y + 16, "Vertical Sync:");
  blitCoolString(CFG_X, CFG_Y + 32, "Play Sound:");

  char string[80];
  sprintf(string, "%d", gameconfig.delay_rate / 1000L);
  blitString(CFG_X + 120, CFG_Y,      15, string);

  blitString(CFG_X + 120, CFG_Y + 16, 15,
             (gameconfig.vertical_sync ? "Yes" : "No"));

  blitString(CFG_X + 120, CFG_Y + 32, 15,
             (gameconfig.play_sound ? "Yes" : "No"));

  fadeIn(gamePalette);

  int curchoice = 0, oldchoice = 0, done = 0;
  while (!done)
  {
    oldchoice = curchoice;

    int command = 0;

    while (keyPressed()) {}
    int key = 0;
    while (!key)
    {
      if (keyTable[UPARROW])    { key = 1; if (--curchoice == -1) curchoice = 0; }
      if (keyTable[DOWNARROW])  { key = 1; if (++curchoice == 3)  curchoice = 2; }
      if (keyTable[RIGHTARROW]) { key = 1; command   = 1;  }
      if (keyTable[LEFTARROW])  { key = 1; command   = -1; }
      if (keyTable[ESC])        { key = 1; done      = 1;  }
    }
    PlaySound(SOUND_CHOICE);

    clearArea(CFG_X - 16, CFG_Y - 4 + oldchoice * 16, 16, 16);
    blitImage(CFG_X - 16, CFG_Y - 4 + curchoice * 16, &arrow);

    if (command == 1)
    {
      if (curchoice == 2)
        gameconfig.play_sound = 1;
      else
      if (curchoice == 1)
        gameconfig.vertical_sync = 1;
      else
      {
        gameconfig.delay_rate += 1000L;
        if (gameconfig.delay_rate > 60000L) gameconfig.delay_rate = 60000L;
      }
    }

    if (command == -1)
    {
      if (curchoice == 2)
        gameconfig.play_sound = 0;
      else
      if (curchoice == 1)
        gameconfig.vertical_sync = 0;
      else
      {
        gameconfig.delay_rate -= 1000L;
        if (gameconfig.delay_rate < 0L) gameconfig.delay_rate = 0L;
      }
    }

    clearArea(CFG_X + 120, CFG_Y, 80, 96);
    sprintf(string, "%d", gameconfig.delay_rate / 1000L);
    blitString(CFG_X + 120, CFG_Y,      15, string);

    blitString(CFG_X + 120, CFG_Y + 16, 15,
               (gameconfig.vertical_sync ? "Yes" : "No"));

    blitString(CFG_X + 120, CFG_Y + 32, 15,
               (gameconfig.play_sound ? "Yes" : "No"));
  }

  fadeOut(gamePalette);
}

//////////////////////////////////////////////////////////////////////////////

#define NUM_CHOICES 4
#define MENU_X      26
#define MENU_Y      10

void mainMenu(void)
{
  char *menuItems[NUM_CHOICES] = { "New Game", "Resume Game",
                                   "Configure", "Quit" };
  int curChoice = 0, oldChoice = 0;

  int quit = 0;
  while (!quit)
  {
    curBuf = screen;
    clearScreen();

    window(MENU_X - 16, MENU_Y, 104, NUM_CHOICES * 16);

    int x;
    for (x = 0; x < NUM_CHOICES; x++)
      blitCoolString(MENU_X, MENU_Y + 4 + x * 16, menuItems[x]);

    fadeIn(gamePalette);

    int done = 0;
    while (!done)
    {
      blitImage(MENU_X - 16, MENU_Y + curChoice * 16, &arrow);

      if (oldChoice != curChoice) PlaySound(SOUND_CHOICE);
      oldChoice = curChoice;
      while (keyPressed()) {}
      
      int key = 0;
      while (!key)
      {
        if (keyTable[UPARROW])
        {
          if (--curChoice == -1) curChoice = NUM_CHOICES - 1;
          key = 1;
        }

        if (keyTable[DOWNARROW])
        {
          if (++curChoice == NUM_CHOICES) curChoice = 0;
          key = 1;
        }

        if (keyTable[ENTER])
          { done = 1; key = 1; }
      }

      clearArea(MENU_X - 16, MENU_Y + oldChoice * 16, 16, 16);
    }

    PlaySound(SOUND_SELECT);

    fadeOut(gamePalette);

    gamestatus = NORMAL;
    switch (curChoice)
    {
      case 0: newGame(START); break;
      case 1: newGame(LOAD); break;
      case 2: configure(); break;
      case 3: quit = 1;
    }
  }
}

//////////////////////////////////////////////////////////////////////////////

int mode;

int Area;
int numEntities;
entityType entity[MAX_ENTITIES];

#define pmode      entity[i].data.person.mode
#define pmapx      entity[i].data.person.mapx
#define pmapy      entity[i].data.person.mapy
#define pdirection entity[i].data.person.direction
#define pcurDelay  entity[i].data.person.curDelay
#define pdelay     entity[i].data.person.delay
#define poffx      entity[i].data.person.offx
#define poffy      entity[i].data.person.offy
#define plean      entity[i].data.person.lean
#define pspriteSet entity[i].data.person.spriteSet
#define pmessage   entity[i].data.person.message
#define pspeed     entity[i].data.person.speed

#define PLAYER     0
#define ENTITY     1

#define cmode      entity[i].data.chest.mode
#define cmapx      entity[i].data.chest.mapx
#define cmapy      entity[i].data.chest.mapy
#define cspriteSet entity[i].data.chest.spriteSet
#define ctype      entity[i].data.chest.type
#define cnum       entity[i].data.chest.num
#define cid        entity[i].data.chest.id

#define CLOSED     0
#define OPENED     1

#define imapx      entity[i].data.inn.mapx
#define imapy      entity[i].data.inn.mapy
#define ispriteSet entity[i].data.inn.spriteSet
#define iname      entity[i].data.inn.name
#define icost      entity[i].data.inn.cost

#define smapx      entity[i].data.shop.mapx
#define smapy      entity[i].data.shop.mapy
#define sspriteSet entity[i].data.shop.spriteSet
#define smessage   entity[i].data.shop.message
#define sitems     entity[i].data.shop.items

#define bmapx      entity[i].data.boss.mapx
#define bmapy      entity[i].data.boss.mapy
#define bspriteSet entity[i].data.boss.spriteSet
#define bmessage   entity[i].data.boss.message

byte empty(int x, int y, byte user)
{
  int i;

  if (!walkable[mapTable(x, y)])
    return 0;

  if (user == ENTITY)
    for (i = 0; i < subAreaData[Area].numExits; i++)
    {
      if (subAreaData[Area].exit[i].x == x &&
          subAreaData[Area].exit[i].y == y)
            return 0;
    }

  for (i = 0; i < numEntities; i++)
  {
    if (entity[i].type == PERSON)
    {
      if (x == pmapx && y == pmapy) return 0;
      if (pmode == MOVING)
      {
        if (pdirection == UP    && x == pmapx && y == pmapy - 1) return 0; else
        if (pdirection == DOWN  && x == pmapx && y == pmapy + 1) return 0; else
        if (pdirection == LEFT  && x == pmapx - 1 && y == pmapy) return 0; else
        if (pdirection == RIGHT && x == pmapx + 1 && y == pmapy) return 0;
      }
    }

    if (entity[i].type == CHEST)
      if (x == cmapx && y == cmapy) return 0;

    if (entity[i].type == INN)
      if (x == imapx && y == imapy) return 0;

    if (entity[i].type == SHOP)
      if (x == smapx && y == smapy) return 0;

    if (entity[i].type == BOSS)
      if (x == bmapx && y == bmapy) return 0;
  }

  if (x == mapx && y == mapy)
    return 0;

  if (mode == MOVING)
  {
    if (direction == UP    && x == mapx && y == mapy - 1) return 0; else
    if (direction == DOWN  && x == mapx && y == mapy + 1) return 0; else
    if (direction == LEFT  && x == mapx - 1 && y == mapy) return 0; else
    if (direction == RIGHT && x == mapx + 1 && y == mapy) return 0;
  }

  return 1;
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

#define LINE_WIDTH 36
#define TX         14
#define TY         12

int lx, ly;

void sendword(char *parse_word)
{
  int string_length = strlen(parse_word);

  if (string_length + lx > LINE_WIDTH)
    { lx = 0; ly++; }

  blitCoolString(TX + lx * 8, TY + ly * 10, parse_word);
  lx += string_length + 1;

  PlaySound(SOUND_WORDBEEP);
  HTimer tt;
  tt.timerOn();
  while (tt.getElapsed() < 100000L) {}
  tt.timerOff();
}

int message_start;

int getword(char *parse_word, char far *message)
{
  memset(parse_word, 0, LINE_WIDTH + 1);

  int word_off = 0;                // Offset into parsed word

  if (message[message_start] == 0)   // If end of message, return 1
    return 1;

  if (message[message_start] == ' ')  // Return a blank word
    { message_start++; return 0; }

  // Loop until SPACE or NULL is found (or word is bigger than LINE_WIDTH)
  while ( (message[message_start + word_off] != ' ') &&
          (message[message_start + word_off] != 0  )    )
  {
    parse_word[word_off] = message[message_start + word_off];
    word_off++;

    if (word_off >= LINE_WIDTH)
      break;
  }

  message_start += word_off + 1;

  return 0;
}

void talkMessage(char far *message)
{
  char parse[LINE_WIDTH + 1];

  message_start = lx = ly = 0;

  byte far *oldbuf;

  oldbuf = curBuf;
  curBuf = screen;

  window(10, 10, 309 - 10, 189 - 10);

  while (!getword(parse, message))
    sendword(parse);

  while (keyPressed())  {}
  while (!keyPressed()) {}

  curBuf = oldbuf;
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

void openChest(int i)
{
  if (cmode == CLOSED)
  {
    byte far *oldbuf = curBuf;
    curBuf = screen;

    PlaySound(SOUND_CHEST);

    window(10, 10, 300, 40);
    blitCoolString(10, 10, "You have opened a chest!");
    blitCoolString(10, 18, "You got...");

    char string[31];

    switch (ctype)
    {
      case BLANK:  strcpy(string, "Nothing!");                   break;
      case ITEM:   strcpy(string, gamedata.itemData[cnum].name); break;
      case WEAPON: strcpy(string, gamedata.weapons [cnum].name); break; 
      case SHIELD: strcpy(string, gamedata.shields [cnum].name); break;
      case HELMET: strcpy(string, gamedata.shields [cnum].name); break;
      case MAIL:   strcpy(string, gamedata.mails   [cnum].name); break;
      case GOLD:   sprintf(string, "%ld GP", cnum);              break;
    }

    blitString(10, 30, 15, string);

    if (ctype == GOLD)
      player.gold += cnum;
    else
      if (ctype != BLANK)
      {
        if (additem(ctype, cnum))
        {
#define toomuch "No room"
          window(40, 40, strlen(toomuch) * 8 + 4, 12);
          blitCoolString(42, 42, toomuch);
        }
        else
        {
          cmode = OPENED;
          player.chests[player.numchests] = cid;
          player.numchests++;
        }
      }
    while (keyPressed()) {}
    while (!keyPressed()) {}

    curBuf = oldbuf;
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

void inn(int i)
{
  byte far *oldbuf = curBuf;
  curBuf = screen;

  window(10, 10, 319 - 20, 80);

  blitCoolString(12, 12, "Welcome to the");

  char string[80];
  sprintf(string, "%s. (%d GP)", iname, icost);
  blitString(12, 24, 15, string);

  blitCoolString(12, 40, "Would you like to stay?");

  blitString(28, 60, 15, "Yes   No");
  blitImage(58, 54, &arrow);

  int curChoice = 1;
  while (keyTable[ENTER] == 0)
  {
    if (keyTable[LEFTARROW])
    {
      clearArea(12, 54, 16, 16); clearArea(58, 54, 16, 16);
      blitImage(12, 54, &arrow);
      curChoice = 0;
      PlaySound(SOUND_CHOICE);
      while (keyTable[LEFTARROW]) {}
    }

    if (keyTable[RIGHTARROW])
    {
      clearArea(12, 54, 16, 16); clearArea(58, 54, 16, 16);
      blitImage(58, 54, &arrow);
      curChoice = 1;
      PlaySound(SOUND_CHOICE);
      while (keyTable[RIGHTARROW]) {}
    }
  }

  if (curChoice == 0)
  {
    if (player.gold >= icost)
    {
      player.gold -= icost;
      player.HP   =  player.maxHP;
      player.MP   =  player.maxMP;

      PlaySound(SOUND_INN);
      fadeOut(gamePalette, 100000);

      HTimer t;
      t.timerOn();
      while (t.getElapsed() < 1000000L) {}
      t.timerOff();

      window(10, 10, 319 - 20, 80);
      blitString(12, 12, 15, "Thank you for staying.");

      fadeIn(gamePalette, 100000);
    }
    else
    {
      window(10, 10, 319 - 20, 80);
      blitString(12, 12, 15, "You don't have the money!");
      blitString(12, 24, 15, "Go rid the area of monsters.");
    }
  }
  else
  {
    window(10, 10, 319 - 20, 80);
    blitString(12, 12, 15, "Okay.  Come back later.");
  }

  while (keyPressed())  {}
  while (!keyPressed()) {}

  curBuf = oldbuf;
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

struct tempListType
{
  char name[13];
  int cost;
};

#define gd gamedata
#define sn sitems[x].num

void buy(int i)
{
  /* Generates Temporary List */
  tempListType tlist[6];

  int x;
  for (x = 0; x < 6; x++)
  {
    switch (sitems[x].type)
    {
      case BLANK:  memcpy(tlist[x].name, "", 13);
                   tlist[x].cost = 0; break;
      case ITEM:   memcpy(tlist[x].name, gd.itemData[sn].name, 13);
                   tlist[x].cost = gd.itemData[sn].cost; break;
      case WEAPON: memcpy(tlist[x].name, gd.weapons[sn].name, 13);
                   tlist[x].cost = gd.weapons[sn].cost; break;
      case SHIELD: memcpy(tlist[x].name, gd.shields[sn].name, 13);
                   tlist[x].cost = gd.shields[sn].cost; break;
      case HELMET: memcpy(tlist[x].name, gd.helmets[sn].name, 13);
                   tlist[x].cost = gd.helmets[sn].cost; break;
      case MAIL:   memcpy(tlist[x].name, gd.mails[sn].name, 13);
                   tlist[x].cost = gd.mails[sn].cost; break;
    }
  }

  window(10, 40, 319 - 20, 56);

  char string[80];
  for (x = 0; x < 6; x++)
  {
    sprintf(string, "%-12s %-4d", tlist[x].name, tlist[x].cost);
    blitString(26, 44 + x * 8, 15, string);
  }
  blitImage(10, 40, &arrow);

  window(40, 140, 12 * 8 + 4, 12);
  sprintf(string, "Gold: %ld", player.gold);
  blitString(42, 142, 15, string);

  int curchoice = 0, oldchoice = 0;

  int done = 0;
  while (!done)
  {
    if (oldchoice != curchoice) PlaySound(SOUND_CHOICE);
    oldchoice = curchoice;
    while (keyPressed()) {}
    int key = 0, exec = 0;
    while (!key)
    {
      if (keyTable[UPARROW])   { key = 1; if (--curchoice == -1) curchoice = 0; }
      if (keyTable[DOWNARROW]) { key = 1; if (++curchoice == 6)  curchoice = 5; }
      if (keyTable[ESC])       { key = 1; done = 1; PlaySound(SOUND_SELECT); }
      if (keyTable[ENTER])     { key = 1; exec = 1; PlaySound(SOUND_SELECT); }
    }

    clearArea(10, 40 + 8 * oldchoice, 16, 16);    

    if (exec)
    {
      window(40, 56, 40, 24);
      blitString(56, 60, 15, "Yes");
      blitString(56, 68, 15, "No");
      blitImage(40, 64, &arrow);

      while (keyPressed()) {}

      int query = 1;
      while (!keyTable[ENTER])
      {
        if (keyTable[UPARROW])   if (query == 1)
          { clearArea(40, 64, 16, 16); blitImage(40, 56, &arrow); query = 0; }
        if (keyTable[DOWNARROW]) if (query == 0)
          { clearArea(40, 56, 16, 16); blitImage(40, 64, &arrow); query = 1; }
      }

      if (query == 0)
      {
        if (player.gold >= tlist[curchoice].cost)
        {
          player.gold -= tlist[curchoice].cost;
          if (additem(sitems[curchoice].type, sitems[curchoice].num))
          {
#define noroom "No room"
            window(50, 66, strlen(noroom) * 8 + 4, 12);
            blitString(52, 68, 15, noroom);
          }
        }
        else
        {
#define nogold "Not enough gold!"
          /* Not enough gold! */
          window(50, 66, strlen(nogold) * 8 + 4, 12);
          blitString(52, 68, 15, nogold);
        }
        while (keyPressed())  {}
        while (!keyPressed()) {}
      }

      /* Redraw everything that the previous window destroyed */
      window(10, 40, 319 - 20, 56);
      for (x = 0; x < 6; x++)
      {
        sprintf(string, "%-12s %-4d", tlist[x].name, tlist[x].cost);
        blitString(26, 44 + x * 8, 15, string);
      }

      window(40, 140, 12 * 8 + 4, 12);
      sprintf(string, "Gold: %ld", player.gold);
      blitString(42, 142, 15, string);
    }

    blitImage(10, 40 + 8 * curchoice, &arrow);
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

#define items(x)    gamedata.items[x]
#define itemData(x) gamedata.itemData[x]
#define weapons(x)  gamedata.weapons[x]
#define shields(x)  gamedata.shields[x]
#define helmets(x)  gamedata.helmets[x]
#define mails(x)    gamedata.mails[x]

void listItems(int topItem)
{
  char string[80];

  window(10, 40, 319 - 20, 106);
  blitString(12, 42, 15, "Sell");

  int x;
  for (x = topItem; x < topItem + 10; x++)
  {
    int itemnum = items(x).num;
    switch (items(x).type)
    {
      case BLANK:  string[0] = 0; break;
      case ITEM:   sprintf(string, "%-12s   %ld", itemData(itemnum).name,
                           (long)itemData(itemnum).cost / 2); break;
      case WEAPON: sprintf(string, "%-12s   %ld", weapons(itemnum).name,
                           (long)weapons(itemnum).cost / 2);  break;
      case SHIELD: sprintf(string, "%-12s   %ld", shields(itemnum).name,
                           (long)shields(itemnum).cost / 2);  break;
      case HELMET: sprintf(string, "%-12s   %ld", helmets(itemnum).name,
                           (long)helmets(itemnum).cost / 2);  break;
      case MAIL:   sprintf(string, "%-12s   %ld", mails(itemnum).name,
                           (long)mails(itemnum).cost / 2);  break;
    }
    blitString(26, (x - topItem) * 8 + 58, 15, string);
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

void sell(void)
{
  listItems(0);
  blitImage(10, 54, &arrow);

  int topitem = 0, curitem = 0;

  int done = 0;
  while (!done)
  {
    int key = 0, exec = 0;

    listItems(topitem);
    blitImage(10, 54 + (curitem - topitem) * 8, &arrow);

    while (keyPressed()) {}
    while (!key)
    {
      if (keyTable[UPARROW])   { key = 1; if (--curitem == topitem - 1)
                                 if (--topitem == -1)
                                   { topitem++; curitem++; }
                                 PlaySound(SOUND_CHOICE);
                               }
      if (keyTable[DOWNARROW]) { key = 1; if (++curitem == topitem + 10)
                                 if (++topitem == NUM_ITEMS)
                                   { topitem--; curitem--; }
                                 PlaySound(SOUND_CHOICE);
                               }
      if (keyTable[ESC])       { key = 1; done = 1; PlaySound(SOUND_SELECT); }
      if (keyTable[ENTER])     { key = 1; exec = 1; PlaySound(SOUND_SELECT); }
    }

    if (exec && items(curitem).type != BLANK)
    {
      window(40, 56, 40, 24);
      blitString(56, 60, 15, "Yes");
      blitString(56, 68, 15, "No");
      blitImage(40, 64, &arrow);

      while (keyPressed()) {}

      int query = 1;
      while (!keyTable[ENTER])
      {
        if (keyTable[UPARROW])   if (query == 1)
          { clearArea(40, 64, 16, 16); blitImage(40, 56, &arrow); query = 0;
            PlaySound(SOUND_CHOICE); }
        if (keyTable[DOWNARROW]) if (query == 0)
          { clearArea(40, 56, 16, 16); blitImage(40, 64, &arrow); query = 1;
            PlaySound(SOUND_CHOICE); }
      }

      if (query == 0)
      {
        int itemnum = items(curitem).num;

        int sellprice;
        switch (items(curitem).type)
        {
          case ITEM:   sellprice = (long)itemData(itemnum).cost / 2; break;
          case WEAPON: sellprice = (long)weapons (itemnum).cost / 2; break;
          case SHIELD: sellprice = (long)shields (itemnum).cost / 2; break;
          case HELMET: sellprice = (long)helmets (itemnum).cost / 2; break;
          case MAIL:   sellprice = (long)mails   (itemnum).cost / 2; break;
        }
        player.gold += sellprice;

        items(curitem).type = BLANK;
      }
    }
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

void shop(int i)
{
  byte far *oldbuf = curBuf;
  curBuf = screen;

  talkMessage(messages[smessage].txt);
  blitScreen(mainbuf);

  window(10, 10, 150, 16);
  blitString(16 + 12,                           14, 15, "Buy");
  blitString(16 + 12 + 3 * 8 + 16,              14, 15, "Sell");
  blitString(16 + 12 + 3 * 8 + 16 + 4 * 8 + 16, 14, 15, "Leave");
  blitImage(10, 10, &arrow);

  int curchoice = 0, oldchoice = 0;

  int done = 0;
  while (!done)
  {
    if (oldchoice != curchoice) PlaySound(SOUND_CHOICE);
    oldchoice = curchoice;
    while (keyPressed()) {}
    int key = 0, exec = 0;
    while (!key)
    {
      if (keyTable[LEFTARROW])  { key = 1; if (--curchoice == -1) curchoice = 0; }
      if (keyTable[RIGHTARROW]) { key = 1; if (++curchoice == 3)  curchoice = 2; }
      if (keyTable[ENTER])      { key = 1; exec = 1; PlaySound(SOUND_SELECT); }
      if (keyTable[ESC])        { key = 1; done = 1; PlaySound(SOUND_SELECT); }
    }

    switch (oldchoice)
    {
      case 0: clearArea(10,                           10, 16, 16); break;
      case 1: clearArea(10 + 3 * 8 + 16,              10, 16, 16); break;
      case 2: clearArea(10 + 3 * 8 + 16 + 4 * 8 + 16, 10, 16, 16); break;
    }

    if (exec)
    {
      if (curchoice == 0) buy(i);
      if (curchoice == 1) sell();
      if (curchoice == 2) done = 1;

      blitScreen(mainbuf);
      window(10, 10, 150, 16);
      blitString(16 + 12,                           14, 15, "Buy");
      blitString(16 + 12 + 3 * 8 + 16,              14, 15, "Sell");
      blitString(16 + 12 + 3 * 8 + 16 + 4 * 8 + 16, 14, 15, "Leave");
    }

    switch (curchoice)
    {
      case 0: blitImage(10,                           10, &arrow); break;
      case 1: blitImage(10 + 3 * 8 + 16,              10, &arrow); break;
      case 2: blitImage(10 + 3 * 8 + 16 + 4 * 8 + 16, 10, &arrow); break;
    }
  }

  switch (oldchoice)
  {
    case 0: clearArea(10,                           10, 16, 16); break;
    case 1: clearArea(10 + 3 * 8 + 16,              10, 16, 16); break;
    case 2: clearArea(10 + 3 * 8 + 16 + 4 * 8 + 16, 10, 16, 16); break;
  }

  blitScreen(mainbuf);
  while (keyPressed()) {}

  curBuf = oldbuf;
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

void ending(void)
{
  PCX_PICTURE image;

  if (PCX_Init(&image))
    errorAbort("Error: Not enough memory\n\r");
  if (PCX_Load(ENDING1_FILE, &image))
    { PCX_Close(&image); errorAbort("Error: Could not open ending1 file\n\r"); }

  colorType blank = { 0, 0, 0 };
  for (int index = 0; index < 256; index++)
    setPalette(index, &blank);

  blitScreen(image.buffer);
  PCX_Close(&image);

  fadeIn((colorType *)image.palette, INTRO_FADE);
  while (keyPressed())  {}
  while (!keyPressed()) {}
  PlaySound(SOUND_SELECT);
  fadeOut((colorType *)image.palette, INTRO_FADE);


  if (PCX_Init(&image))
    errorAbort("Error: Not enough memory\n\r");
  if (PCX_Load(ENDING2_FILE, &image))
    { PCX_Close(&image); errorAbort("Error: Could not open ending2 file\n\r"); }

  blitScreen(image.buffer);
  PCX_Close(&image);

  fadeIn((colorType *)image.palette, INTRO_FADE);
  while (keyPressed())  {}
  while (!keyPressed()) {}
  PlaySound(SOUND_ENDING);
  fadeOut((colorType *)image.palette, INTRO_FADE);
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

void fightboss(int i)
{
  talkMessage(messages[bmessage].txt);
  blitScreen(mainbuf);

  // Last battle type should have one monster which is the last boss.
  battle(NUM_BATTLES - 1, mapTable(mapx, mapy));

  if (player.HP < 1)
    { gamestatus = PLAYER_DEAD; return; }

  ending();

  gamestatus = ENDING;
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

void talk(int x, int y)
{
  int i;
  for (i = 0; i < numEntities; i++)
  {
    switch (entity[i].type)
    {
      case PERSON:
        if (x == pmapx && y == pmapy)
          { talkMessage(messages[pmessage].txt); return; }
        break;

      case CHEST:
        if (x == cmapx && y == cmapy)
          { openChest(i); return; }
        break;

      case INN:
        if (x == imapx && y == imapy)
          { inn(i); return; }
        break;

      case SHOP:
        if (x == smapx && y == smapy)
          { shop(i); return; }
        break;

      case BOSS:
        if (x == bmapx && y == bmapy)
          { fightboss(i); return; }
    }
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

void moveEntity(int i)
{
  if (pdirection == UP)    poffy--; else
  if (pdirection == DOWN)  poffy++; else
  if (pdirection == LEFT)  poffx--; else
  if (pdirection == RIGHT) poffx++;
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

void subArea(int area, int enter_direction)
{
  register i;
  int oldmapx = mapx, oldmapy = mapy;

  loadMap(subAreaData[area].mapFile, map, battleMap);
  mapx = subAreaData[area].startx;
  mapy = subAreaData[area].starty;

  Area = area;
  numEntities = subAreaData[area].numEntities;
  for (i = 0; i < numEntities; i++)
  {
    entity[i] = subAreaData[area].entity[i];
    if (entity[i].type == PERSON)
      { pmode = WAITING; pdelay++; pcurDelay = 0; poffx = 0; poffy = 0; }
    if (entity[i].type == CHEST)
    {
      cmode = CLOSED;

      int x;
      for (x = 0; x < player.numchests; x++)
        if (player.chests[x] == cid)
          cmode = OPENED;
    }
  }

  curBuf = mainbuf;                     // Draw to buffer
  drawMap(0, 0);                        // Draw map
  blitSprite(150, 90, direction);       // Draw player sprite in center

  for (i = 0; i < numEntities; i++)
  {
    if (entity[i].type == PERSON)
      { blitSprite((pmapx - mapx) * 20 + 150, (pmapy - mapy) * 20 + 90,
                   pspriteSet * 12 + pdirection); }

    if (entity[i].type == CHEST)
      { blitSprite((cmapx - mapx) * 20 + 150, (cmapy - mapy) * 20 + 90,
                   cspriteSet + cmode); }

    if (entity[i].type == INN)
      { blitSprite((imapx - mapx) * 20 + 150, (imapy - mapy) * 20 + 90,
                   ispriteSet); }

    if (entity[i].type == SHOP)
      { blitSprite((smapx - mapx) * 20 + 150, (smapy - mapy) * 20 + 90,
                   sspriteSet); }

    if (entity[i].type == BOSS)
      { blitSprite((bmapx - mapx) * 20 + 150, (bmapy - mapy) * 20 + 90,
                   bspriteSet); }
  }

  blitScreen(mainbuf);               // Copy buffer to screen
  fadeIn(gamePalette);

  HTimer maptimer;

  mode = WAITING;
  int oldmode;
  int offx = 0;
  int offy = 0;
  int numexit;

  int leftArea = 0;
  while (!leftArea)
  {
    for (i = 0; i < numEntities; i++)
    {
      if (entity[i].type == PERSON)
      {
        if (pmode == WAITING)
        {

#define oldwalk()   { pdirection = oldDirection; pmode = WAITING; \
                      pcurDelay = pdelay - 1; }

          pcurDelay++;
          if (pcurDelay == pdelay)
          {
            int oldDirection = pdirection;
            pcurDelay = 0;

            if (!random(3))
              pdirection = random(4) * 3;

            if (pdirection == UP && !empty(pmapx, pmapy - 1, ENTITY))
              oldwalk() else
            if (pdirection == DOWN && !empty(pmapx, pmapy + 1, ENTITY))
              oldwalk() else
            if (pdirection == LEFT && !empty(pmapx - 1, pmapy, ENTITY))
              oldwalk() else
            if (pdirection == RIGHT && !empty(pmapx + 1, pmapy, ENTITY))
              oldwalk() else
            pmode = MOVING;
          }
        }
        else
        {
          switch (pspeed)
          {
            case 0:
              if (pcurDelay == 4)
              { moveEntity(i); pcurDelay = 0; }
              else pcurDelay++;
              break;
            case 1:
              if (pcurDelay == 2)
              { moveEntity(i); pcurDelay = 0; }
              else pcurDelay++;
              break;
            case 2:
              if (pcurDelay == 1)
              { moveEntity(i); pcurDelay = 0; }
              else pcurDelay++;
              break;
            case 4: moveEntity(i);
            case 3: moveEntity(i);
          }

          if (poffy == -20)
          { pmapy--; pmode = WAITING; poffy = 0; pcurDelay = 0; } else
          if (poffy == 20)
          { pmapy++; pmode = WAITING; poffy = 0; pcurDelay = 0; } else
          if (poffx == -20)
          { pmapx--; pmode = WAITING; poffx = 0; pcurDelay = 0; } else
          if (poffx == 20)
          { pmapx++; pmode = WAITING; poffx = 0; pcurDelay = 0; }

          if (pmapx == -1)         pmapx = MAP_X_SIZE - 1; else
          if (pmapx == MAP_X_SIZE) pmapx = 0;

          if (pmapy == -1)         pmapy = MAP_Y_SIZE - 1; else
          if (pmapy == MAP_Y_SIZE) pmapy = 0;
        }

        if (pdirection == UP)
        { if (poffy > -5 || poffy < -15) plean = 0; else
            plean = (pmapx + pmapy) % 2 + 1; } else
        if (pdirection == DOWN)
        { if (poffy < 5 || poffy > 15) plean = 0; else
            plean = (pmapx + pmapy) % 2 + 1; } else
        if (pdirection == LEFT)
        { if (poffx > -5 || poffx < -15) plean = 0; else
            plean = (pmapx + pmapy) % 2 + 1; } else
        if (pdirection == RIGHT)
        { if (poffx < 5 || poffx > 15) plean = 0; else
            plean = (pmapx + pmapy) % 2 + 1; }
      }
    }

    if (mode == WAITING)
    {
      if (keyTable[UPARROW])
      {
        direction = UP;
        if (empty(mapx, mapy - 1, PLAYER)) mode = MOVING;
      } else

      if (keyTable[DOWNARROW])
      {
        direction = DOWN;
        if (empty(mapx, mapy + 1, PLAYER)) mode = MOVING;
      } else

      if (keyTable[LEFTARROW])
      {
        direction = LEFT;
        if (empty(mapx - 1, mapy, PLAYER)) mode = MOVING;
      } else

      if (keyTable[RIGHTARROW])
      {
        direction = RIGHT;
        if (empty(mapx + 1, mapy, PLAYER)) mode = MOVING;
      }

      if (keyTable[SPACEBAR])
      {
        switch (direction)
        {
          case UP:    talk(mapx, mapy - 1); break;
          case DOWN:  talk(mapx, mapy + 1); break;
          case LEFT:  talk(mapx - 1, mapy); break;
          case RIGHT: talk(mapx + 1, mapy);
        }

        if (gamestatus == PLAYER_DEAD) return;
        if (gamestatus == ENDING)      return;
      }

      if (keyTable[ESC])
      {
        menu(1);
        if (gamestatus == MENU_QUIT) return;
      }
    }

    oldmode = mode;

    if (mode == MOVING)
    {
      if (direction == UP)    offy--; else
      if (direction == DOWN)  offy++; else
      if (direction == LEFT)  offx--; else
      if (direction == RIGHT) offx++;

      if (offy == -20) { mapy--; mode = WAITING; offy = 0; } else
      if (offy == 20)  { mapy++; mode = WAITING; offy = 0; } else
      if (offx == -20) { mapx--; mode = WAITING; offx = 0; } else
      if (offx == 20)  { mapx++; mode = WAITING; offx = 0; }

      if (mapx == -1)          mapx = MAP_X_SIZE - 1; else
      if (mapx == MAP_X_SIZE)  mapx = 0;
      if (mapy == -1)          mapy = MAP_Y_SIZE - 1; else
      if (mapy == MAP_Y_SIZE)  mapy = 0;
    }

#define bd battleData[btype]

    if (oldmode != mode)
    {
      int btype;
      btype = bTable(mapx, mapy);
      if (btype)
        battle(btype, mapTable(mapx, mapy));

        // battle() checks whether the battle actually occurs
        // with random numbers and which monster is fought.

      if (player.HP < 1)
        { pause(); gamestatus = PLAYER_DEAD; return; }
    }

    for (i = 0; i < subAreaData[area].numExits; i++)
    {
      if (mapx == subAreaData[area].exit[i].x &&
          mapy == subAreaData[area].exit[i].y)
        { leftArea = 1; numexit = i; }
    }

    if (mapTable(mapx, mapy) > (128 - NUM_SUB_AREAS))
    {
      fadeOut(gamePalette);

      int subarea = 127 - mapTable(mapx, mapy);
      subArea(subarea, direction);

      loadMap(subAreaData[area].mapFile, map, battleMap);

      numEntities = subAreaData[area].numEntities;
      for (i = 0; i < numEntities; i++)
      {
        entity[i] = subAreaData[area].entity[i];
        if (entity[i].type == PERSON)
          { pmode = WAITING; pdelay++; pcurDelay = 0; poffx = 0; poffy = 0; }
        if (entity[i].type == CHEST)
        {
          cmode = CLOSED;

          int x;
          for (x = 0; x < player.numchests; x++)
            if (player.chests[x] == cid)
              cmode = OPENED;
        }
      }

      drawMap(0, 0);
      blitSprite(150, 90, direction);

      for (i = 0; i < numEntities; i++)
      {
        if (entity[i].type == PERSON)
          { blitSprite((pmapx - mapx) * 20 + 150, (pmapy - mapy) * 20 + 90, pspriteSet * 12 + pdirection); }

        if (entity[i].type == CHEST)
          { blitSprite((cmapx - mapx) * 20 + 150, (cmapy - mapy) * 20 + 90, cspriteSet + cmode); }

        if (entity[i].type == INN)
          { blitSprite((imapx - mapx) * 20 + 150, (imapy - mapy) * 20 + 90, ispriteSet); }

        if (entity[i].type == SHOP)
          { blitSprite((smapx - mapx) * 20 + 150, (smapy - mapy) * 20 + 90, sspriteSet); }

        if (entity[i].type == BOSS)
          { blitSprite((bmapx - mapx) * 20 + 150, (bmapy - mapy) * 20 + 90, bspriteSet); }
      }

      blitScreen(mainbuf);
      fadeIn(gamePalette);

      if (gamestatus == PLAYER_DEAD) return;
      if (gamestatus == MENU_QUIT)   return;
      if (gamestatus == ENDING)      return;
    }

    int lean;
    if (direction == UP)
    { if (offy > -5 || offy < -15) lean = 0; else
        lean = (mapx + mapy) % 2 + 1; } else
    if (direction == DOWN)
    { if (offy < 5 || offy > 15) lean = 0; else
        lean = (mapx + mapy) % 2 + 1; } else
    if (direction == LEFT)
    { if (offx > -5 || offx < -15) lean = 0; else
        lean = (mapx + mapy) % 2 + 1; } else
    if (direction == RIGHT)
    { if (offx < 5 || offx > 15) lean = 0; else
        lean = (mapx + mapy) % 2 + 1; }

    maptimer.timerOn();
    while (maptimer.getElapsed() < gameconfig.delay_rate) {}
    maptimer.timerOff();

    curBuf = mainbuf;
    drawMap(-offx, -offy);
    blitSprite(150, 90, direction + lean);

    for (i = 0; i < numEntities; i++)
    {
      if (entity[i].type == PERSON)
        blitSprite((pmapx - mapx) * 20 + 150 - offx + poffx,
                   (pmapy - mapy) * 20 + 90 - offy + poffy,
                   pspriteSet * 12 + pdirection + plean);
      else
      if (entity[i].type == CHEST)
        blitSprite((cmapx - mapx) * 20 + 150 - offx,
                   (cmapy - mapy) * 20 + 90  - offy,
                   cspriteSet + cmode);
      else
      if (entity[i].type == INN)
        blitSprite((imapx - mapx) * 20 + 150 - offx,
                   (imapy - mapy) * 20 + 90  - offy,
                   ispriteSet);
      else
      if (entity[i].type == SHOP)
        blitSprite((smapx - mapx) * 20 + 150 - offx,
                   (smapy - mapy) * 20 + 90  - offy,
                   sspriteSet);
      else
      if (entity[i].type == BOSS)
        blitSprite((bmapx - mapx) * 20 + 150 - offx,
                   (bmapy - mapy) * 20 + 90  - offy,
                   bspriteSet);
    }

    if (gameconfig.vertical_sync) vsync();
    blitScreen(mainbuf);
  }

  mapx = oldmapx; mapy = oldmapy;

  switch (subAreaData[area].exit[numexit].type)
  {
    case 0:
      switch (enter_direction)
      {
        case UP:    mapy++; break;
        case DOWN:  mapy--; break;
        case LEFT:  mapx++; break;
        case RIGHT: mapx--; break;
      }
      break;
    case 1:
      switch (subAreaData[area].exit[numexit].dir)
      {
        case 0: mapy++; break;
        case 1: mapx++; break;
        case 2: mapy--; break;
        case 3: mapx--; break;
      }
      break;
    case 2:
      mapx = subAreaData[area].exit[numexit].lx;
      mapy = subAreaData[area].exit[numexit].ly;
      break;
  }

  fadeOut(gamePalette);
}

//////////////////////////////////////////////////////////////////////////////

char gstr[80];

void getstring(char *str, char *prompt, unsigned length)
{
  memset(str, 0, length + 1);

  int x = 10, y = 10;
  char c;
  int index = 0;

  blitString(x, y, 15, prompt);
  blitCoolString(x + strlen(prompt) * 8, y, "_");

  x += strlen(prompt) * 8;

  while (1)
  {
    switch (c = getch())
    {
      case 0: getch(); break;
      case 8: if (index) str[--index] = 0; break;
      case 13:
        clearArea(x + strlen(str), 8, 8, 8, 0);
        return;

      default:
        if (c >= ' ' && c <= '~' && index < length)
        {
          str[index] = c;
          index++;
        }
    }

    clearArea(x, y, strlen(str) * 8 + 16, 8, 0);
    sprintf(gstr, "%s_", str);
    blitCoolString(x, y, gstr);
  }
}

//////////////////////////////////////////////////////////////////////////////

void newGame(int where)
{
  int oldmapx, oldmapy;
  int index;


  if (where == START)
  {
    byte far *oldbuf = curBuf;

    curBuf = screen;
    clearScreen();

    fadeIn(gamePalette);

    closeKeyboard();
    getstring(player.name, "Enter player's name: ", 12);
    initKeyboard();

    fadeOut(gamePalette);

    curBuf = oldbuf;

    PCX_PICTURE image;

    if (PCX_Init(&image))
      errorAbort("Error: Not enough memory\n\r");
    if (PCX_Load(START1_FILE, &image))
      { PCX_Close(&image); errorAbort("Error: Could not open start1 file\n\r"); }

    blitScreen(image.buffer);
    PCX_Close(&image);

    fadeIn((colorType *)image.palette, INTRO_FADE);
    while (keyPressed())  {}
    while (!keyPressed()) {}
    PlaySound(SOUND_SELECT);
    fadeOut((colorType *)image.palette, INTRO_FADE);

    if (PCX_Init(&image))
      errorAbort("Error: Not enough memory\n\r");
    if (PCX_Load(START2_FILE, &image))
      { PCX_Close(&image); errorAbort("Error: Could not open start2 file\n\r"); }

    blitScreen(image.buffer);
    PCX_Close(&image);

    fadeIn((colorType *)image.palette, INTRO_FADE);
    while (keyPressed())  {}
    while (!keyPressed()) {}
    PlaySound(SOUND_NEWGAME);
    fadeOut((colorType *)image.palette, INTRO_FADE);


    /* Intro over.  Start game. */

    gamestatus = NORMAL;

    mapx = 49; mapy = 49; direction = DOWN;

    player.HP = player.maxHP = 15;
    player.MP = player.maxMP = 5;

    for (index = 0; index < NUM_SPELLS; index++)
      player.spells[index] = 0;

    player.numchests = 0;
    for (index = 0; index < NUM_SUB_AREAS * MAX_ENTITIES; index++)
      player.chests[index] = 0;

    player.weapon = UNEQUIPPED;
    player.shield = UNEQUIPPED;
    player.helmet = UNEQUIPPED;
    player.mail   = UNEQUIPPED;

    player.strength = 2;
    player.speed    = 1;
    player.wisdom   = 1;

    player.gold = 0;
    player.exp = 0;
    player.level = 0;
  }
  else
  {
    FILE *stream = fopen(SAVEGAME_FILE, "rb");
    if (stream == NULL) return;
    fread(&player, 1, sizeof(playerType), stream);
    fread(gamedata.items, 1, sizeof(gamedata.items), stream);
    fread(&mapx, 1, sizeof(mapx), stream);
    fread(&mapy, 1, sizeof(mapy), stream);
    fclose(stream);
  }

  loadMap(MAP_FILE, map, battleMap);

  clearScreen();

  curBuf = mainbuf;
  drawMap(0, 0);
  blitSprite(150, 90, direction);
  blitScreen(mainbuf);

  fadeIn(gamePalette);

  while (1)
  {
    oldmapx = mapx;
    oldmapy = mapy;

    if (keyTable[UPARROW]    && walkable[mapTable(mapx, mapy - 1)])
      { slideMap(0, -1); mapy--; }
    else
    if (keyTable[DOWNARROW]  && walkable[mapTable(mapx, mapy + 1)])
      { slideMap(0, 1);  mapy++; }
    else
    if (keyTable[LEFTARROW]  && walkable[mapTable(mapx - 1, mapy)])
      { slideMap(-1, 0); mapx--; }
    else
    if (keyTable[RIGHTARROW] && walkable[mapTable(mapx + 1, mapy)])
      { slideMap(1, 0);  mapx++; }

    if (mapx == -1)          mapx = MAP_X_SIZE - 1; else
    if (mapx == MAP_X_SIZE)  mapx = 0;
    if (mapy == -1)          mapy = MAP_Y_SIZE - 1; else
    if (mapy == MAP_Y_SIZE)  mapy = 0;

    if (oldmapx != mapx || oldmapy != mapy)
    {
      int btype;
      btype = bTable(mapx, mapy);
      if (btype)
      {
        battle(btype, mapTable(mapx, mapy));
        // battle() checks whether the battle actually occurs
        // with random numbers and which monster is fought.

        if (player.HP < 1) // If player died in battle, return to main menu
        {
          fadeOut(gamePalette);
          return;
        }
      }
    }

    if (mapTable(mapx, mapy) > (128 - NUM_SUB_AREAS))
    {
      PlaySound(SOUND_SUBAREA);
      fadeOut(gamePalette);

      int subarea = 127 - mapTable(mapx, mapy);
      subArea(subarea, direction);

      if (gamestatus == PLAYER_DEAD)
      {
        gamestatus = NORMAL;
        fadeOut(gamePalette);
        return;
      }

      if (gamestatus == MENU_QUIT)
      {
        gamestatus = NORMAL;
        fadeOut(gamePalette);
        return;
      }

      if (gamestatus == ENDING)
      {
        gamestatus = NORMAL;
        return;
      }

      loadMap(MAP_FILE, map, battleMap);

      drawMap(0, 0);
      blitSprite(150, 90, direction);
      blitScreen(mainbuf);

      fadeIn(gamePalette);
    }

    if (keyTable[ESC])
    {
      menu(0);
      if (gamestatus == MENU_QUIT)
      {
        gamestatus = NORMAL;
        fadeOut(gamePalette);
        return;
      }
    }
  }
}
